INCLUDE "rst.asm"
INCLUDE "interrupts.asm"

SECTION "High Home", ROM0
INCLUDE "home/highhome.asm"
INCLUDE "home/crash.asm"

	ds 2

SECTION "Version", ROM0
BuildNumber:: bigdw BUILD_NUMBER
VersionNumber::
	db VERSION_MAJOR
	db VERSION_MINOR

SECTION "Header", ROM0

Start::
	nop
	jp _Start

	rept ($150 - $104)
		db $00 ; zero-filling the header for rgbfix, since it clashes with $ff padding otherwise
	endr

SECTION "Home", ROM0

INCLUDE "home/init.asm"
INCLUDE "home/vblank.asm"
INCLUDE "home/delay.asm"
INCLUDE "home/rtc.asm"
INCLUDE "home/fade.asm"
INCLUDE "home/lcd.asm"
INCLUDE "home/time.asm"
INCLUDE "home/serial.asm"
INCLUDE "home/joypad.asm"
INCLUDE "home/decompress.asm"
INCLUDE "home/palettes.asm"
INCLUDE "home/copy.asm"
INCLUDE "home/text.asm"
INCLUDE "home/video.asm"
INCLUDE "home/map_objects.asm"
INCLUDE "home/sine.asm"
INCLUDE "home/movement.asm"
INCLUDE "home/tilemap.asm"
INCLUDE "home/menu.asm"
INCLUDE "home/handshake.asm"
INCLUDE "home/game_time.asm"
INCLUDE "home/stopwatch.asm"
INCLUDE "home/map.asm"
INCLUDE "home/count_events.asm"
INCLUDE "home/predef.asm" ;keep close to farcall (within reach of jr)
INCLUDE "home/farcall.asm"
INCLUDE "home/window.asm"
INCLUDE "home/flag.asm"
INCLUDE "home/jumptable.asm"
INCLUDE "home/bankswitch.asm"
INCLUDE "home/string.asm"
INCLUDE "home/item.asm"
INCLUDE "home/random.asm"
INCLUDE "home/math.asm"
INCLUDE "home/stone_queue.asm"
INCLUDE "home/trainer.asm"
INCLUDE "home/pokedex_flags.asm"
INCLUDE "home/get_names.asm"
INCLUDE "home/cry.asm"
INCLUDE "home/battle.asm"
INCLUDE "home/audio.asm"
INCLUDE "home/ded.asm"
INCLUDE "home/hp_pal.asm"
INCLUDE "home/sprites.asm"
INCLUDE "home/scripting.asm"
INCLUDE "home/weekday.asm"
INCLUDE "home/pokemon_data.asm"
INCLUDE "home/vwf.asm"
INCLUDE "home/print_text.asm"

IsInArray::
; Find value a for every e bytes in array hl.
; Return index in b and carry if found.

	ld b, 0
	ld d, b
	ld c, a
	jr .handleLoop

.loop
	inc b
	add hl, de
.handleLoop
	ld a, [hl]
	cp -1
	ret z ; saves 3 bytes over conditionally jumping to an `and a \ ret` here
	cp c
	jr nz, .loop
; in array
	scf
	ret

IsHMMove::
	ld hl, HMMoves
	; fallthrough

IsInSingularArray::
; Find value a in array hl
; Return index in b and carry if found
	ld b, l ; save original count in b
	ld d, $ff
	ld e, a
.loop
	ld a, [hli]
	cp d
	jr z, .notInArray
	cp e
	jr nz, .loop
; in array
	dec hl
	ld a, l
	sub b ; subtract the current offset with the original offset to get the index
	ld b, a
	scf
	ret

.notInArray
	and a
	ret

ScrollingMenu::
	call CopyMenuData2
	anonbankpush _ScrollingMenu

	call _InitScrollingMenu
	call ScrollingMenu_UpdatePalettes
	call _ScrollingMenu
	ld a, [wMenuJoypad]
	ret

CountSetBits::
; Count the number of set bits in b bytes starting from hl.
; Return in a, c and [wd265].

	xor a
.next
	ld e, [hl]
	ld d, 8

.count
	srl e
	adc 0
	dec d
	jr nz, .count

	inc hl
	dec b
	jr nz, .next

	ld c, a
	ld [wd265], a
	ret

InitScrollingMenu::
	ld a, [wMenuBorderTopCoord]
	and a
	jr z, .skipDec1
	dec a
.skipDec1
	ld b, a
	ld a, [wMenuBorderBottomCoord]
	cp SCREEN_HEIGHT
	jr c, .validBorderBottom
	ld a, SCREEN_HEIGHT - 1
.validBorderBottom
	sub b
	ld d, a
	ld a, [wMenuBorderLeftCoord]
	and a
	jr z, .skipDec2
	dec a
.skipDec2
	ld c, a
	ld a, [wMenuBorderRightCoord]
	cp SCREEN_WIDTH
	jr c, .validBorderRight
	ld a, SCREEN_WIDTH - 1
.validBorderRight
	sub c
	ld e, a
	push de
	call Coord2Tile
	pop bc
	jp TextBox

RequestLYOverrides::
	ld a, [wLCDCPointer]
	and a
	ret z

	ld a, LOW(wLYOverridesBackup)
	ldh [hRequestedVTileSource], a
	ld a, HIGH(wLYOverridesBackup)
	ldh [hRequestedVTileSource + 1], a

	ld a, LOW(wLYOverrides)
	ldh [hRequestedVTileDest], a
	ld a, HIGH(wLYOverrides)
	ldh [hRequestedVTileDest + 1], a

	ld a, (wLYOverridesEnd - wLYOverrides) / 16
	ldh [hLYOverrideStackCopyAmount], a
	ret

CancelMapSign::
	ld a, $80
	ld [wMapSignRoutineIdx], a
	xor a
	ld [wMapSignTimer], a
	ld a, SCREEN_HEIGHT_PX
	ldh [rWY], a
	ldh [hWY], a
	ret

DMGCompatBGMapTransfer::
; transfer 6 rows instead of 9 rows so it'll be normal speed compatible
	ldh a, [hBGMapMode]
	push af
	ld a, 5
	ldh [hBGMapMode], a
	inc a ; 6
	ldh [hTilesPerCycle], a
	xor a
	ldh [hBGMapHalf], a
	call DelayFrame
	ld a, 6
	ldh [hBGMapHalf], a
	call DelayFrame
	ld a, 12
	ldh [hBGMapHalf], a
	call DelayFrame
	pop af
	ldh [hBGMapMode], a
	ret

LoadFontsBattleExtra::
	jpba _LoadFontsBattleExtra
